package com.sillycycle.bagleyd.ant3d;

/* Code for selection spliced in is due to Alex Shvets */
/* http://home.comcast.net/~shvets/Programs/Examples/GameLife/life2.html */
// Ant3DPopup.java

import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;

import com.sillycycle.bagleyd.util.Box;

public class Ant3DPopup implements ClipboardOwner, ActionListener {
	Clipboard clipboard = new Clipboard("Life");
	Ant3DCanvas ant3d;
	static final boolean DEBUG = false;

	public Ant3DPopup(Ant3DCanvas ant3d) {
		this.ant3d = ant3d;
	}

	@Override
	public void lostOwnership(Clipboard lostClipboard, Transferable contents) {
		// implements ClipboardOwner
	}

	// implements ActionListener
	@Override
	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		Box selectedBox = getSelectedBox();
		Ant3DMatrix field = ant3d.getField();

		if (DEBUG) {
			if (selectedBox == null)
				System.out.println("actionPeformed: selectedBox = null");
			else if (ant3d.getField() == null)
				System.out.println("actionPeformed: field = null");
		}
		if (selectedBox == null || field == null)
			return;
		if (DEBUG) {
			System.out.println("actionPeformed: " + command);
		}
		ant3d.setCursor(new Cursor(Cursor.WAIT_CURSOR));
		switch(command) {
		case "Cut":
			doCut(selectedBox);
			break;
		case "Copy":
			doCopy(selectedBox);
			break;
		case "Paste":
			doPaste(selectedBox);
			break;
		case "Clear":
			doClear(selectedBox);
			break;
		case "Cw":
			doRotate(selectedBox, 0, true);
			break;
		case "Ccw":
			doRotate(selectedBox, 0, false);
			break;
		case "Refl Wall":
			doReflect(selectedBox, 0, true);
			break;
		case "Refl Grnd":
			doReflect(selectedBox, 0, false);
			break;
		default:
		}
		ant3d.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	}

	public Box getSelectedBox() {
		return ant3d.getSelectedBox();
	}

	public void doClear(Box selectedBox) {
		Ant3DMatrix field = ant3d.getField();

		field.clearSubMatrix(selectedBox);
		ant3d.redrawField(); //(selectedBox);
	}

	public void doCopy(Box selectedBox) {
		Ant3DMatrix field = ant3d.getField();

		Ant3DMatrix selection = field.getSubMatrix(selectedBox);
		Ant3DSelection data = new Ant3DSelection(selection);
		clipboard.setContents(data, this);
	}

	public void doCut(Box selectedBox) {
		Ant3DMatrix field = ant3d.getField();

		Ant3DMatrix selection = field.getSubMatrix(selectedBox);
		Ant3DSelection data = new Ant3DSelection(selection);
		clipboard.setContents(data, this);

		field.clearSubMatrix(selectedBox);
		ant3d.redrawField(); //(selectedBox);
	}

	public void doPaste(Box selectedBox) {
		Ant3DMatrix field = ant3d.getField();

		try {
			Transferable data = clipboard.getContents(this);
			if (data == null) {
				return;
			}
			Ant3DMatrix selection = (Ant3DMatrix) data.getTransferData(
				Ant3DSelection.lifeFlavor);

			selectedBox.setSize(selection.getColumns(),
				selection.getRows());
			field.setSubMatrix(selection, selectedBox);
			ant3d.redrawField(); //(selectedBox);
		} catch (UnsupportedFlavorException | IOException e) {
			e.printStackTrace();
		}
	}

	public void doRotate(Box selectedBox, int plane, boolean cw) {
		Ant3DMatrix field = ant3d.getField();

		try {
			Transferable data = clipboard.getContents(this);
			if (data == null) {
				return;
			}
			Ant3DMatrix selection = (Ant3DMatrix) data.getTransferData(
				Ant3DSelection.lifeFlavor);

			selectedBox.setSize(selection.getColumns(),
				selection.getRows());
			field.rotate(selection, selectedBox,
				ant3d.getPolyhedron(), plane, cw);
			ant3d.redrawField(); //(selectedBox);
		} catch (UnsupportedFlavorException | IOException e) {
			e.printStackTrace();
		}
	}

	public void doReflect(Box selectedBox, int plane, boolean wall) {
		Ant3DMatrix field = ant3d.getField();

		try {
			Transferable data = clipboard.getContents(this);
			if (data == null) {
				return;
			}
			Ant3DMatrix selection = (Ant3DMatrix) data.getTransferData(
				Ant3DSelection.lifeFlavor);

			selectedBox.setSize(selection.getColumns(),
				selection.getRows());
			field.reflect(selection, selectedBox,
				ant3d.getPolyhedron(), plane, wall);
			ant3d.redrawField(); //(selectedBox);
		} catch (UnsupportedFlavorException | IOException e) {
			e.printStackTrace();
		}
	}
}
