package com.sillycycle.bagleyd.util;

/*
 * Combo Box coded by David Bagley
 */

import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

/**
 * The <code>ComboBox</code> class combines a JComboBox and
 * a JLabel in one widget.
 *
 * @author David Bagley
 */

public class ComboBox extends JPanel {
	private static final long serialVersionUID = 42L;
	JComboBox<Object> comboBox;
	JLabel label;
	String string;
	transient Object[] objects;
	int value = 0;

	public ComboBox(String labelString, Object[] group) {
		setLayout(new BorderLayout());
		this.string = "  " + labelString;
		this.objects = group;
		this.label = new JLabel(this.string, SwingConstants.CENTER);
		add(this.label, BorderLayout.PAGE_START);
		this.comboBox = new JComboBox<>(this.objects);
		add(this.comboBox, BorderLayout.CENTER);
	}

	public void addActionListener(ActionListener l) {
		this.comboBox.addActionListener(l);
	}

	public synchronized int getSelectedItem() {
		Object object = this.comboBox.getSelectedItem();
		int objectValue = -1;

		for (int i = 0; i < this.objects.length; i++) {
			if (object.equals(this.objects[i])) {
				objectValue = i;
				break;
			}
		}
		this.value = objectValue;
		return objectValue;
	}

	public synchronized void setSelectedItem(int value) {
//System.out.println("ComboBox.setSelectedItem: " + value + " " + objects[value]);
		if (value >= 0 && value < this.objects.length) {
			this.value = value;
			this.comboBox.setSelectedItem(this.objects[value]);
//System.out.println("ComboBox.setSelectedItem set: " + value + " " + objects[value]);
		}
	}

	public void updateList(Object[] objectList, int objectValue) {
		this.comboBox.removeAllItems();
		this.objects = objectList;
		for (Object object : objectList) {
//System.out.println("ComboBox.updateList: " + objectList.length + " " + object);
			this.comboBox.addItem(object);
		}
		setSelectedItem(objectValue);
//System.out.println("ComboBox.updateList: " + objectValue);
	}
}
